@@ -1,4 +1,4 @@
-/* $Id: bogoconfig.c,v 1.230 2005/06/29 10:05:22 m-a Exp $ */
+/* $Id: bogoconfig.c,v 1.234 2005/11/05 04:04:40 relson Exp $ */
 
 /*****************************************************************************
 
@@ -47,7 +47,6 @@ Note: bogolexer also uses configfile.c.
 #include "bogoreader.h"
 #include "bool.h"
 #include "charset.h"
-#include "configfile.h"
 #include "datastore.h"
 #include "datastore_db.h"
 #include "error.h"
@@ -170,6 +169,8 @@ static bool get_double(const char *name, const char *arg, double *d)
 static char *get_string(const char *name, const char *arg)
 {
     char *s = xstrdup(arg);
+    remove_comment(s);
+
     if (DEBUG_CONFIG(2))
 	fprintf(dbgout, "%s -> '%s'\n", name, s);
     return s;
@@ -715,7 +716,10 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 #ifndef	DISABLE_TRANSACTIONS
 	if (!dsm_options_bogofilter(option, name, val))
 #endif
-	    abort();
+	{
+	    fprintf(stderr, "Invalid option '%s'.\n", name);
+	    exit(EX_ERROR);
+	}
     }
 }
 
