@@ -1,4 +1,4 @@
-/* $Id: bogoutil.c,v 1.208 2005/04/07 22:27:00 m-a Exp $ */
+/* $Id: bogoutil.c,v 1.231 2005/12/27 15:38:12 relson Exp $ */
 
 /*****************************************************************************
 
@@ -35,6 +35,7 @@ AUTHORS:
 #include "msgcounts.h"
 #include "paths.h"
 #include "prob.h"
+#include "rand_sleep.h"
 #include "robx.h"
 #include "sighandler.h"
 #include "swap.h"
@@ -67,7 +68,8 @@ static void ds_open_failure(bfpath *bfp, void *dbe)
     fprintf(stderr, "Error accessing file or directory '%s'.\n", bfp->filepath);
     if (errno != 0)
 	fprintf(stderr, "error #%d - %s.\n", errno, strerror(errno));
-    ds_cleanup(dbe);
+    if (dbe != NULL)
+	ds_cleanup(dbe);
     exit(EX_ERROR);
 }
 
@@ -88,13 +90,13 @@ static int ds_dump_hook(word_t *key, dsv_t *data,
     if (replace_nonascii_characters)
 	do_replace_nonascii_characters(key->text, key->leng);
 
-    printf( "%.*s %lu %lu",
+    fprintf( fpo, "%.*s %lu %lu",
 	    CLAMP_INT_MAX(key->leng), key->text,
 	    (unsigned long)data->spamcount,
 	    (unsigned long)data->goodcount);
     if (data->date)
-	printf(" %lu", (unsigned long)data->date);
-    putchar('\n');
+	fprintf( fpo, " %lu", (unsigned long)data->date);
+    fprintf( fpo, "\n");
 
     fflush(stdout); /* solicit ferror flag if output is shorter than buffer */
     return ferror(stdout) ? 1 : 0;
@@ -140,6 +142,7 @@ static byte *spanword(byte *t)
 static bool is_count(const char *in)
 {
     static const char *const msgc = MSG_COUNT;
+    static const char *const enco = WORDLIST_ENCODING;
 
     /* anything that doesn't start with a . is a count */
     if (in[0] != '.')
@@ -147,6 +150,10 @@ static bool is_count(const char *in)
     /* .MSG_COUNT is also a count */
     if (strcmp(in, msgc) == 0)
 	return true;
+    /* .ENCODING is also a count */
+    if (strcmp(in, enco) == 0)
+	return true;
+
     return false;
 }
 
@@ -348,7 +355,7 @@ static ex_t display_words(bfpath *bfp, int argc, char **argv, bool show_probabil
 	robx = ROBX;
     }
 
-    printf(head_format, "", "spam", "good", "  Fisher");
+    fprintf(fpo, head_format, "", "spam", "good", "  Fisher");
     while (argc >= 0)
     {
 	dsv_t val;
@@ -378,11 +385,11 @@ static ex_t display_words(bfpath *bfp, int argc, char **argv, bool show_probabil
 		good_count = val.goodcount;
 
 		if (!show_probability)
-		    printf(data_format, token->text, spam_count, good_count);
+		    fprintf(fpo, data_format, token->text, spam_count, good_count);
 		else
 		{
 		    rob_prob = calc_prob(good_count, spam_count, msgcnts.goodcount, msgcnts.spamcount);
-		    printf(data_format, token->text, spam_count, good_count, rob_prob);
+		    fprintf(fpo, data_format, token->text, spam_count, good_count, rob_prob);
 		}
 		break;
 	    case 1:
@@ -426,11 +433,21 @@ static ex_t get_robx(bfpath *bfp)
 	dsv_t val;
 	word_t *word_robx = word_news(ROBX_W);
 
+	/* since compute_robinson_x() closes the wordlists, 
+	   init_wordlist() must be called again */
+	init_wordlist("word", bfp->filepath, 0, WL_REGULAR);
+
 	open_wordlists(DS_WRITE);
 
 	val.goodcount = 0;
 	val.spamcount = (uint32_t) (rx * 1000000);
-	ret = ds_write(word_lists->dsh, word_robx, &val);
+	do {
+	    ret = ds_write(word_lists->dsh, word_robx, &val);
+	    if (ret == DS_ABORT_RETRY) {
+		rand_sleep(1000, 1000000);
+		begin_wordlist(word_lists);
+	    }
+	} while (ret == DS_ABORT_RETRY);
 
 	close_wordlists(true);
 	free_wordlists();
@@ -443,10 +460,10 @@ static ex_t get_robx(bfpath *bfp)
 
 static void print_version(void)
 {
-    (void)fprintf(stderr,
+    (void)fprintf(stdout,
 		  "%s version %s\n"
 		  "    Database: %s\n"
-		  "Copyright (C) 2002-2004 Gyepi Sam, David Relson, Matthias Andree\n\n"
+		  "Copyright (C) 2002-2005 Gyepi Sam, David Relson, Matthias Andree\n\n"
 		  "%s comes with ABSOLUTELY NO WARRANTY.  "
 		  "This is free software, and\nyou are welcome to "
 		  "redistribute it under the General Public License.  "
@@ -456,22 +473,26 @@ static void print_version(void)
 		  progname, version, ds_version_str(), PACKAGE);
 }
 
-static void usage(void)
+static void usage(FILE *fp)
 {
-    fprintf(stderr, "Usage: %s {-h|-V}\n", progname);
-    fprintf(stderr, "   or: %s [OPTIONS] {-d|-l|-u|-m|-w|-p|--db-verify} file%s\n",
+    fprintf(fp, "Usage: %s {-h|-V}\n", progname);
+    fprintf(fp, "   or: %s [OPTIONS] {-d|-l|-u|-m|-w|-p|--db-verify} file%s\n",
 	    progname, DB_EXT);
-    fprintf(stderr, "   or: %s [OPTIONS] {-H|-r|-R} file\n", progname);
+    fprintf(fp, "   or: %s [OPTIONS] {-H|-r|-R} file\n", progname);
 #if defined (ENABLE_DB_DATASTORE) || defined (ENABLE_SQLITE_DATASTORE)
-    fprintf(stderr, "   or: %s [OPTIONS] {--db-print-pagesize} file%s\n",
+    fprintf(fp, "   or: %s [OPTIONS] {--db-print-leafpage-count} file%s\n",
+	    progname, DB_EXT);
+    fprintf(fp, "   or: %s [OPTIONS] {--db-print-pagesize} file%s\n",
 	    progname, DB_EXT);
 #endif
 #if	defined(ENABLE_DB_DATASTORE) && !defined(DISABLE_TRANSACTIONS)
-    fprintf(stderr, "   or: %s [OPTIONS] {--db-checkpoint} directory\n",
+    fprintf(fp, "   or: %s [OPTIONS] {--db-checkpoint} directory\n",
 	    progname);
-    fprintf(stderr, "   or: %s [OPTIONS] {--db-prune|--db-remove-environment} directory\n",
+    fprintf(fp, "   or: %s [OPTIONS] {--db-list-logfiles} directory [list options]\n",
 	    progname);
-    fprintf(stderr, "   or: %s [OPTIONS] {--db-recover|--db-recover-harder} directory\n",
+    fprintf(fp, "   or: %s [OPTIONS] {--db-prune|--db-remove-environment} directory\n",
+	    progname);
+    fprintf(fp, "   or: %s [OPTIONS] {--db-recover|--db-recover-harder} directory\n",
 	    progname);
 #endif
 }
@@ -499,15 +520,14 @@ static const char *help_text[] = {
     "\n",
 
     "info options:\n",
-    "  -w file_or_dir              - display counts for words from stdin.\n",
-    "  -p file_or_dir              - display word counts and probabilities.\n",
+    "  -w file                     - display counts for words from stdin.\n",
+    "  -p file                     - display word counts and probabilities.\n",
     "  -I, --input-file=file       - read 'file' instead of standard input.\n",
-    "  -H dir                      - display histogram and statistics for wordlist.\n",
+    "  -H file                     - display histogram and statistics for wordlist.\n",
     "                                - use with -v  to exclude hapaxes.\n",
     "                                - use with -vv to exclude pure spam/ham.\n",
     "  -r file                     - compute Robinson's X for the specified file.\n",
     "  -R file                     - compute Robinson's X and save it in wordlist.\n",
-
     "\n",
 
     "database maintenance, the \"-m file\" option is required in this group:\n",
@@ -517,22 +537,25 @@ static const char *help_text[] = {
     "  -c cnt                      - exclude tokens with lower counts.\n",
     "  -s l,h                      - exclude tokens with lengths between 'l' and 'h'\n"
     "                                (low and high).\n",
+#ifndef	DISABLE_UNICODE
+    "  --unicode=yes/no            - convert wordlist to/from unicode\n",
+#endif
     "\n",
     NULL
     };
 
-static void help(void)
+static void help(FILE *fp)
 {
     uint i;
     const char **messages;
-    usage();
+    usage(fp);
     messages = help_text;
     for (i=0; messages[i] != NULL; i++)
-	(void)fprintf(stderr, "%s", messages[i]);
+	(void)fprintf(fp, "%s", messages[i]);
     messages = dsm_help_bogoutil();
     for (i=0; messages[i] != NULL; i++)
-	(void)fprintf(stderr, "%s", messages[i]);
-    (void)fprintf(stderr,
+	(void)fprintf(fp, "%s", messages[i]);
+    (void)fprintf(fp,
 		  "%s (version %s) is part of the bogofilter package.\n",
                   progname, version
 	);
@@ -548,9 +571,14 @@ static struct option longopts_bogoutil[] = {
     LONGOPTIONS_COMMON
     /* longoptions.h - options for bogofilter and bogoutil */
     LONGOPTIONS_DB
+    /* longoptions.h - options for bogolexer and bogoutil */
+    LONGOPTIONS_LEX_UTIL
+
     /* bogoutil specific options */
     { "db-prune",                       R, 0, O_DB_PRUNE },
     { "db-checkpoint",                  R, 0, O_DB_CHECKPOINT },
+    { "db-list-logfiles",               R, 0, O_DB_LIST_LOGFILES },
+    { "db-print-leafpage-count",	R, 0, O_DB_PRINT_LEAFPAGE_COUNT },
     { "db-print-pagesize",		R, 0, O_DB_PRINT_PAGESIZE },
     { "db-recover",                     R, 0, O_DB_RECOVER },
     { "db-recover-harder",              R, 0, O_DB_RECOVER_HARDER },
@@ -561,7 +589,7 @@ static struct option longopts_bogoutil[] = {
     { NULL,				0, 0, 0 }
 };
 
-#define	OPTIONS	":a:c:Cd:DhH:I:k:l:m:np:r:R:s:u:vVw:x:X:y:"
+#define	OPTIONS	":a:c:Cd:DhH:I:k:l:m:nO:p:r:R:s:u:vVw:x:X:y:"
 
 static int process_arglist(int argc, char **argv)
 {
@@ -569,6 +597,7 @@ static int process_arglist(int argc, char **argv)
     int count = 0;
 
     fpin = stdin;
+    fpo  = stdout;
     dbgout = stderr;
 
 #ifdef __EMX__
@@ -594,7 +623,7 @@ static int process_arglist(int argc, char **argv)
 
     if (count != 1)
     {
-	usage();
+	usage(stderr);
 	fprintf(stderr, "%s: Exactly one of the file or directory commands must be present.\n", progname);
 	exit(EX_ERROR);
     }
@@ -652,6 +681,12 @@ static int process_arg(int option, const char *name, const char *val)
 	ds_file = val;
 	break;
 
+    case O_DB_PRINT_LEAFPAGE_COUNT:
+	flag = M_LEAFPAGES;
+	count += 1;
+	ds_file = val;
+	break;
+
     case O_DB_PRINT_PAGESIZE:
 	flag = M_PAGESIZE;
 	count += 1;
@@ -682,7 +717,7 @@ static int process_arg(int option, const char *name, const char *val)
 	exit(EX_ERROR);
 
     case 'h':
-	help();
+	help(stdout);
 	exit(EX_OK);
 
     case 'H':
@@ -755,6 +790,14 @@ static int process_arg(int option, const char *name, const char *val)
 	}
 	break;
 
+    case 'O':
+	fpo = fopen(val, "wt");
+	if (fpo == NULL) {
+	    fprintf(stderr, "Can't write file '%s'\n", val);
+	    exit(EX_ERROR);
+	}
+	break;
+
     case 'D':
 	dbgout = stdout;
 	break;
@@ -765,6 +808,10 @@ static int process_arg(int option, const char *name, const char *val)
 	ds_file = val;
 	break;
 
+    case O_UNICODE:
+	encoding = str_to_bool(val) ? E_UNICODE : E_RAW;
+	break;
+
     default:
 	if (!dsm_options_bogoutil(option, &flag, &count, &ds_file, name, val)) {
 	    fprintf(stderr, "Invalid option '%s'\n", name);
@@ -779,7 +826,7 @@ static bfpath_mode get_mode(cmd_t cmd)
 {
     bfpath_mode mode = BFP_ERROR;
 
-    switch(cmd) {
+    switch (cmd) {
     case M_LOAD:
 	mode = BFP_MAY_CREATE;
 	break;
@@ -789,18 +836,18 @@ static bfpath_mode get_mode(cmd_t cmd)
     case M_ROBX:
     case M_VERIFY:
     case M_WORD:
-	mode = BFP_MUST_EXIST;
-	break;
     case M_CHECKPOINT:	/* database transaction/integrity operations */
     case M_CRECOVER:
+    case M_LEAFPAGES:
     case M_PAGESIZE:
     case M_PURGELOGS:
     case M_RECOVER:
     case M_REMOVEENV:
+    case M_LIST_LOGFILES:
 	mode = BFP_MUST_EXIST;
 	break;
     case M_NONE:
-	usage();
+	usage(stderr);
 	exit(EX_ERROR);
     }
 
@@ -814,7 +861,10 @@ int main(int argc, char *argv[])
     bfpath *bfp;
     bfpath_mode mode;
 
+    fBogoutil = true;
+
     signal_setup();			/* setup to catch signals */
+    atexit(bf_exit);
 
     progtype = build_progtype(progname, DB_TYPE);
 
@@ -824,70 +874,91 @@ int main(int argc, char *argv[])
     process_config_files(false, longopts_bogoutil);	/* need to read lock sizes */
 
     /* Extra or missing parameters */
-    if (flag != M_WORD && argc != optind) {
+    if (flag != M_WORD && flag != M_LIST_LOGFILES && argc != optind) {
 	fprintf(stderr, "Missing or extraneous argument.\n");
-	usage();
+	usage(stderr);
 	exit(EX_ERROR);
     }
 
     bfp = bfpath_create(ds_file);
+    if (bogohome == NULL)
+	set_bogohome( "." );		/* set default */
+
+    bfpath_set_bogohome(bfp);
+
     mode = get_mode(flag);
+    if (bfpath_check_mode(bfp, mode)) {
+	if (bfp->isdir)
+	    bfpath_set_filename(bfp, WORDLIST);
+    }
 
     if (!bfpath_check_mode(bfp, mode)) {
 	fprintf(stderr, "Can't open wordlist '%s'\n", bfp->filepath);
 	exit(EX_ERROR);
     }
 
-    switch(flag) {
+    errno = 0;		/* clear error status */
+
+    switch (flag) {
 	case M_RECOVER:
-	{
 	    ds_init(bfp);
 	    rc = ds_recover(bfp, false);
 	    break;
-	}
 	case M_CRECOVER:
-	{
 	    ds_init(bfp);
 	    rc = ds_recover(bfp, true);
 	    break;
-	}
 	case M_CHECKPOINT:
-	{
 	    ds_init(bfp);
 	    rc = ds_checkpoint(bfp);
 	    break;
-	}
+	case M_LIST_LOGFILES:
+	    dsm_init(bfp);
+	    rc = ds_list_logfiles(bfp, argc - optind, argv + optind);
+	    break;
 	case M_PURGELOGS:
-	{
 	    ds_init(bfp);
 	    rc = ds_purgelogs(bfp);
 	    break;
-	}
 	case M_REMOVEENV:
-	{
 	    dsm_init(bfp);
 	    rc = ds_remove(bfp);
 	    break;
-	}
 	case M_VERIFY:
-	{
 	    dsm_init(bfp);
 	    rc = ds_verify(bfp);
 	    break;
-	}
+	case M_LEAFPAGES:
+	    {
+		u_int32_t c;
+
+		dsm_init(bfp);
+		c = ds_leafpages(bfp);
+		if (c == 0xffffffff) {
+		    fprintf(stderr, "%s: error getting leaf page count.\n", ds_file);
+		    rc = EX_ERROR;
+		} else if (c == 0) {
+		    puts("UNKNOWN");
+		} else {
+		    printf("%lu\n", (unsigned long)c);
+		}
+	    }
+	    break;
 	case M_PAGESIZE:
-	{
-	    u_int32_t s;
-	    dsm_init(bfp);
-	    s = ds_pagesize(bfp);
-	    if (s == 0xffffffff)
-		fprintf(stderr, "%s: error getting page size.\n", ds_file);
-	    else if (s == 0)
-		printf("UNKNOWN\n");
-	    else
-		printf("%lu\n", (unsigned long)s);
+	    {
+		u_int32_t s;
+
+		dsm_init(bfp);
+		s = ds_pagesize(bfp);
+		if (s == 0xffffffff) {
+		    fprintf(stderr, "%s: error getting page size.\n", ds_file);
+		} else if (s == 0) {
+		    puts("UNKNOWN");
+		} else {
+		    printf("%lu\n", (unsigned long)s);
+		}
+	    }
 	    break;
-	}
 	case M_DUMP:
 	    rc = dump_wordlist(bfp);
 	    break;
@@ -916,5 +987,7 @@ int main(int argc, char *argv[])
 	    break;
     }
 
+    bfpath_free(bfp);
+
     return rc;
 }
